/* Simulation of population dynamics in a neighborhood.
   Inspired by The Atlantic Monthly, "Seeing around Corners",
   http://www.theatlantic.com/issues/2002/04/rauch.htm
   and the research of Schelling.
   Copyright 2002, Akkana Peck, akkana@shallowsky.com.
   This is free software: you may use it under the terms of the GPL.
 */

#include "grid.h"
#include "xgrid.h"

#include <stdio.h>
#include <stdlib.h>    // for exit()

// Initial percentages
int VacantPer = 30;   /* percentage of all sites that are vacant */
int BluePer = 65;     /* percentage of populated houses containing blue */
int HappyNum = 1;     /* Number of neighbors of same color required to stay */

#define INT_ARG(arg) \
   if (argc < 2) Usage(progname); \
   arg = atoi(argv[1]); \
   argc -= 2; \
   argv += 2

void Usage(const char* progname)
{
    fprintf(stderr, "Usage: %s [-V] [-c [-t msec]] [-v vacancy%%] [-b blue%%] [-H happy-num] [-x x_cells] [-y y_cells]\n",
            progname);
    exit(1);
}

void ParseArgs(int argc, char **argv)
{
    const char* progname = *argv;
    --argc;
    ++argv;
    while (argc > 0 && argv[0][0] == '-')
    {
        switch (argv[0][1])
        {
          case 'V':
              Verbose = 1;
              --argc;
              ++argv;
              break;
          case 'c':
              Continuous = 1;
              --argc;
              ++argv;
              break;
          case 't':
              INT_ARG(TickTime);
              break;
          case 'v':
              INT_ARG(VacantPer);
              break;
          case 'b':
              INT_ARG(BluePer);
              break;
          case 'H':
              INT_ARG(HappyNum);
              break;
          case 'x':
              INT_ARG(GridX);
              break;
          case 'y':
              INT_ARG(GridY);
              break;
          default:
              Usage(progname);
        }
    }
}

extern void PopulateGrid(int vacantPer, int bluePer);

main(int argc, char **argv)
{
    GridX = 100;
    GridY = 100;

    ParseArgs(argc, argv);

    if (Verbose) {
        printf("Vacancy %d%%\n", VacantPer);
        printf("Blue %d%%, Red %d%%\n", BluePer, 100 - BluePer);
        printf("%d x %d cells\n", GridX, GridY);
    }
    PopulateGrid(VacantPer, BluePer);

    InitWindow();

    /* Initialize colors */
    AllocColor("black");
    AllocColor("blue");
    AllocColor("red");

    grid_main();
}

